/*
   *************************************************** 
    This is a library example for the MLX90614 Temp Sensor
    Written by Limor Fried/Ladyada for Adafruit Industries.  
    BSD license, all text above must be included in any redistribution
   ***************************************************
  
  August, 2020 
  MTS DS3231 Test
  with MLX90614 code
  and microSD
  RTC set to UTC
  Using Timezone to control DST
*/

#include <Wire.h>
#include <Adafruit_MLX90614.h>
#include "DHT.h"        //DHT22
#include <DS1307RTC.h>   // https://github.com/PaulStoffregen/DS1307RTC
#include <Timezone.h>    // https://github.com/JChristensen/Timezone
#include <SPI.h>

#define DHTPIN 5        // DHT data pin
DHT dht(DHTPIN, DHT22);
     
#include <SdFat.h> // Use SdFat so can have .xxxx extension
SdFat SD;

unsigned long LogInterval=300000;  // milliseconds
unsigned long StartTime;  //Log Interval counter
int StartHr = 22;     // 24hr clock. Time to start recording. Probably won't work if after midnight
int StartMin = 40;
int StartTotalmin = StartHr *60 + StartMin;
int FinishHr = 5;     // Time to stop recording
int FinishMin = 45;
int FinishTotalmin = FinishHr *60 + FinishMin;

File MISCFile;
const int chipSelect = 4;  // Chip select for SD

Adafruit_MLX90614 mlx = Adafruit_MLX90614();

// US Mountain Time Zone (Denver)
TimeChangeRule MDT = { "MDT", Second, Sun, Mar, 2, -360 };    //Daylight time = UTC - 6 hours
TimeChangeRule MST = { "MST", First, Sun, Nov, 2, -420 };     //Standard time = UTC - 7 hours
Timezone myTZ(MDT, MST);

TimeChangeRule *tcr;        //pointer to the time change rule, use to get TZ abbrev

// Prototypes
void WriteSerial(int, int, int, int);   // Write to serial port
void WriteSD(int, int, int, int);       // Write to microSD card

void setup(){
  Serial.begin(115200);
//while (!Serial); // M4
  setSyncProvider(RTC.get);   // the function to get the time from the RTC
  if(timeStatus()!= timeSet)
      Serial.println("Unable to sync with the RTC");
  else
      Serial.println("RTC has set the system time");

  mlx.begin();    // start MLX90614  
//  mlx.writeEmissivityReg(63569); // 97% for skin 65535*0.97 = 63569
  dht.begin();    // start DHT22

  if (!SD.begin(chipSelect)){
    Serial.println("Card failed, or not present");
    // don't do anything more:
    while (1);
  }
  MISCFile = SD.open("Night.csv", FILE_WRITE);
  MISCFile.println("Date, Hour, External Temp, Humidity, MLX Temp, IR Temp");
  MISCFile.close();

}
void loop(){
	time_t utc = now();
	time_t local = myTZ.toLocal(utc, &tcr); // This is local time with DST 
 
WriteSerial(month(local), day(local), hour(local), minute(local));
//WriteSD(month(local), day(local), hour(local), minute(local));
delay(1000);
  
  if ( ((hour(local) *60 + minute(local)) >= StartTotalmin) || ((hour(local) *60 + minute(local)) < FinishTotalmin) ){
    if ((millis()-StartTime)>LogInterval){  // Log Interval has passed
      WriteSD(month(local), day(local), hour(local), minute(local));
      StartTime = millis(); //Get StartTime for next LogInterval
    }
  }
}

void WriteSerial(int month, int day, int hour, int minute){   // Write to serial port
  Serial.print(month);
  Serial.print("/");
  Serial.print(day);
  Serial.print("\t");
  Serial.print((hour + minute/60.0),2);
  Serial.print("\tDHT ");
  Serial.print(dht.readTemperature(true)); 
  Serial.print("\tHum "); 
  Serial.print(dht.readHumidity());
  Serial.print("\tMLX ");
  Serial.print(mlx.readAmbientTempF()); 
  Serial.print("\tIR "); 
  Serial.print(mlx.readObjectTempF());
  Serial.println();
}

void WriteSD(int month, int day, int hour, int minute){       // Write to microSD card
  MISCFile = SD.open("Night.csv", FILE_WRITE);
//  MISCFile.println("Date, Hour, External Temp, Humidity, MLX Temp, IR Temp");

    MISCFile.print(month); 
    MISCFile.print('/');
    MISCFile.print(day);
    MISCFile.print(',');
    MISCFile.print((hour + minute/60.0),2);
    MISCFile.print(',');
    MISCFile.print(dht.readTemperature(true),2);  // External Fahrenheit
    MISCFile.print(',');
    MISCFile.print(dht.readHumidity());  // Humidity
    MISCFile.print(',');
    MISCFile.print(mlx.readAmbientTempF(),2);
    MISCFile.print(',');
    MISCFile.println(mlx.readObjectTempF(),2);

    MISCFile.close();
 }
